# array-x.js
Functional methods (inspired by LINQ) on JavaScript arrays


## Examples

```js
var ArrayX: typeof X.Array = require("./index");

var arr = new ArrayX<any>(
    new Date("March 21, 2020"),
    /abc[.]+/g,
    "a", "b", "c",
    1, 2, 3, 4, 5, 6, 7,
    [
        0,
        1
    ],
    [
        2,
        3
    ],
    [
        4,
        5,
        [
            6,
            7,
            [
                8,
                [
                    9,
                    10
                ]
            ]
        ]
    ],
    "last"
);

console.log("first(): " + arr.first());
console.log("last(): " + arr.last());
console.log("skip(2): " + arr.skip(2));
console.log("where(x=>x>3):" + arr.where(x => x > 3));
console.log("select(x=>x.length): " + arr.select(x => x.length));
console.log("flatten(): " + arr.flatten());
console.log("append(): " + arr.append(["a", "b", "c"]));
```

Output:
```
first(): Sat Mar 21 2020 00:00:00 GMT-0700 (Pacific Daylight Time)
last(): last
skip(2): a,b,c,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,8,9,10,last
where(x=>x>3):Sat Mar 21 2020 00:00:00 GMT-0700 (Pacific Daylight Time),4,5,6,7
select(x=>x.length): ,,1,1,1,,,,,,,,2,2,3,4
flatten(): Sat Mar 21 2020 00:00:00 GMT-0700 (Pacific Daylight Time),/abc[.]+/g,a,b,c,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,8,9,10,last
append(): Sat Mar 21 2020 00:00:00 GMT-0700 (Pacific Daylight Time),/abc[.]+/g,a,b,c,1,2,3,4,5,6,7,0,1,2,3,4,5,6,7,8,9,10,last,a,b,c
```

## API

declare var XArray : {
    fromSize: (size?: number) => XArray<any>;
    fromRange: (from: number, to: number) => XArray<any>;
}

declare interface XArray<T> extends Array<T> {
    new<T>(...items) : XArray<T>;

    toArray: () => any[];
    distinct: () => XArray<T>;
    accumulate: (initial: any, accum?: Accumulator) => any;
    union: (array: Array<any>) => XArray<T>;

    select: (func?: Func) => XArray<T>;
    where: (func?: Predicate) => XArray<T>;

    orderBy: (func?: Func) => XArray<T>;
    orderByDesc: (func: Func) => XArray<T>;

    firstOrDefault: (predicate?: Predicate) => T;
    first: (predicate?: Predicate) => T;

    lastOrDefault: (predicate?: Predicate) => T;
    last: (predicate?: Predicate) => T;

    all: (predicate?: Predicate) => boolean;
    any: (predicate?: Predicate) => boolean;

    take: (count: number) => XArray<T>;
    skip: (count: number) => XArray<T>;

    sum: (func?: Func) => number;
    avg: (func?: Func) => number;
    count: (func?: Func) => number;
    min: (func?: Func) => number;
    max: (func?: Func) => number;

    /* 
     * Groups the array values into a dictionary of arrays. 
     * Grouping keys are determined by applying the <func> function arg to each element 
     */
    groupBy: (func) => any;

    /* 
     * Groups this array into an dictionary of <keyCount> arrays,
     * each containing the same number of elements, except possibly the last array, 
     * which may have less elements than the others 
     * if the length of this array is not divisible by <keyCount>
     */
    groupByNumber(keyCount: number): any;

    /*
     * groups this array according to <key> generator function and then 
     * aggregates the resulting dictionary over <value> generator function
     */
    groupAggregate(key, value, aggregation): any;

    removeItem: (item: any) => XArray<T>;
    removeAt: (index: number, count?: number) => XArray<T>;
    insert: (index: number, ...items: any[]) => XArray<T>;
    append: (items: any[]) => XArray<T>;

    flatten: () => XArray<T>;
}

## Install
npm install array-x --save

